@@ -4,8 +4,9 @@ import copy
 import fnmatch
 import json
 from collections.abc import Mapping, Sequence
+from itertools import starmap
 from operator import itemgetter
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, Iterable, List, Optional
 
 import jsonpatch
 import jsonpointer
@@ -18,16 +19,26 @@ def format_json(data: Any, stable: bool = False) -> str:
 
 def apply_json_fragment(
         old: Dict[str, Any],
-        new_fragment: Dict[str, Any],
-        acl: List[str],
+        new_fragment: Dict[str, Any], *,
+        acl: Sequence[str],
+        filters: Sequence[str] | None = None,
 ) -> Dict[str, Any]:
     """
     Replace parts of the old document with 'new_fragment' using ACL restrictions.
+    If `filter` is not `None`, only those parts which also matches at least one filter
+    from the list will be modified (updated or deleted).
     """
     full_new_config = copy.deepcopy(old)
     for acl_item in acl:
         new_pointers = _resolve_json_pointers(acl_item, new_fragment)
         old_pointers = _resolve_json_pointers(acl_item, full_new_config)
+        if filters is not None:
+            new_pointers = _apply_filters_to_json_pointers(
+                new_pointers, filters, content=new_fragment
+            )
+            old_pointers = _apply_filters_to_json_pointers(
+                old_pointers, filters, content=full_new_config
+            )
 
         for pointer in new_pointers:
             new_value = pointer.get(new_fragment)
@@ -97,30 +108,7 @@ def apply_patch(content: Optional[bytes], patch_bytes: bytes) -> bytes:
     return new_contents
 
 
-def apply_acl_filters(content: Dict[str, Any], filters: List[str]) -> Dict[str, Any]:
-    result = {}
-    for f in filters:
-        filter_text = f.strip()
-        if not filter_text:
-            continue
-
-        pointers = _resolve_json_pointers(filter_text, content)
-        for pointer in pointers:
-            part = pointer.get(copy.deepcopy(content))
-
-            sub_tree = result
-            for i in pointer.get_parts():
-                if i not in sub_tree:
-                    sub_tree[i] = {}
-                sub_tree = sub_tree[i]
-
-            patch = jsonpatch.JsonPatch([{"op": "add", "path": pointer.path, "value": part}])
-            result = patch.apply(result)
-
-    return result
-
-
-def _resolve_json_pointers(pattern: str, content: Dict[str, Any]) -> List[jsonpointer.JsonPointer]:
+def _resolve_json_pointers(pattern: str, content: dict[str, Any]) -> list[jsonpointer.JsonPointer]:
     """
     Resolve globbed json pointer pattern to a list of actual pointers, existing in the document.
 
@@ -158,7 +146,7 @@ def _resolve_json_pointers(pattern: str, content: Dict[str, Any]) -> List[jsonpo
     ]
     """
     parts = jsonpointer.JsonPointer(pattern).parts
-    matched = [([], content)]
+    matched = [((), content)]
     for part in parts:
         new_matched = []
         for matched_parts, doc in matched:
@@ -174,10 +162,67 @@ def _resolve_json_pointers(pattern: str, content: Dict[str, Any]) -> List[jsonpo
                     if fnmatch.fnmatchcase(str(i), part)
                 ]
             for key, sub_doc in keys_and_docs:
-                new_matched.append((matched_parts + [key], sub_doc))
+                new_matched.append((matched_parts + (key,), sub_doc))
         matched = new_matched
 
-    ret: List[jsonpointer.JsonPointer] = []
+    ret: list[jsonpointer.JsonPointer] = []
     for matched_parts, _ in matched:
-        ret.append(jsonpointer.JsonPointer("/" + "/".join(matched_parts)))
+        ret.append(jsonpointer.JsonPointer.from_parts(matched_parts))
     return ret
+
+
+def _apply_filters_to_json_pointers(
+        pointers: Iterable[jsonpointer.JsonPointer],
+        filters: Sequence[str], *,
+        content: Any,
+) -> list[jsonpointer.JsonPointer]:
+
+    """
+    Takes a list of pointers, a list of filters and a document
+    and returns a list of pointers that match at least one of the filters
+    (if necessary, pointers may be deeper than from the input).
+
+    For example, given:
+    pointers=["/foo", "/lorem/ipsum", "/lorem/dolor"],
+    filters=["/foo/b*/q*", "/lorem"],
+    content={
+        "foo": {
+            "bar": {
+                "baz": [1, 2],
+                "qux": [3, 4]
+            },
+            "qux": {
+                "baz": [5, 6]
+            }
+        },
+        "lorem": {
+            "ipsum": [7, 8],
+            "dolor": "sit",
+            "amet": "consectetur"
+        }
+    }
+    The function will return:
+    ["/foo/bar/qux", "/lorem/ipsum", "/lorem/dolor"]
+    """
+
+    ret: set[jsonpointer.JsonPointer] = set()
+    for filter_item in filters:
+        filter_parts = jsonpointer.JsonPointer(filter_item).parts
+        for pointer in pointers:
+            pointer_parts = pointer.parts
+            if not all(starmap(fnmatch.fnmatchcase, zip(pointer_parts, filter_parts))):
+                continue  # common part not matched
+            if len(filter_parts) > len(pointer_parts):
+                # filter is deeper than data pointer
+                deeper_doc = pointer.resolve(content)
+                deeper_pattern = "".join((
+                    f"/{jsonpointer.escape(part)}"
+                    for part in filter_parts[len(pointer_parts):]
+                ))
+                ret.update(map(pointer.join, _resolve_json_pointers(deeper_pattern, deeper_doc)))
+            else:
+                ret.add(pointer)
+    # sort return value by some stable key, to decrease the chance
+    # that some bug may lead to unstable output being produced
+    # (since `type(ret) is set`)
+    return sorted(ret, key=str)
